// MAT259 Assignment
////////////////////////////////////////////////////////////////////////////////
// 2D Frequency Mapping: Hans Zimmer's OST and Its Original Film Checkout Trend
//
// Author: Sihwa Park (sihwapark@mat.ucsb.edu)
// 1st Version: 02/01/2018
// Final Version: 03/20/2018
// Copyright (c) 2018 Sihwa Park
////////////////////////////////////////////////////////////////////////////////

Table table;

int rows, columns;
float[][] data;
float[] maxValues, minValues;
String[] titles;

float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;
float lineSegmentLength;
float barMaxHeight;

ArrayList<PVector> linePositions; 
int[] indicesForStartingMonth;

float[] rotationAnglesAtEachStartingPoint;
float radius;
int months;
int padding_months = 24;
String[] startDate;
color bgColor = #7B8D8E;

void setup() {
  
  size(1400, 900);
  smooth();
  surface.setResizable(true);
  
  table = loadTable("HansOSTandMovie.csv");
  rows = table.getRowCount();
  columns = table.getColumnCount();
  println("Rows: " + rows + " Columns: " + columns);
  
  data = new float[columns - 2][rows - 1];
  maxValues = new float[columns - 2];
  minValues = new float[columns - 2];
  titles = new String[columns - 2];
  months = (rows - 1) + padding_months;// * 365;
  indicesForStartingMonth = new int[(columns - 2) / 2];
  rotationAnglesAtEachStartingPoint = new float[(columns - 2) / 2];
  startDate = new String[columns - 2];
  
  for(int i = 0; i < columns - 2; i++) {
    minValues[i] = Float.MAX_VALUE;
    titles[i] = table.getString(0, i + 2);
    for(int j = 0; j < rows - 1; j++) {
      data[i][j] = table.getFloat(j + 1, i + 2);
     
      if(data[i][j] > maxValues[i]) maxValues[i] = data[i][j];
      else if(data[i][j] < minValues[i]) minValues[i] = data[i][j];
    }
  }
  
  for(int i = 0; i < indicesForStartingMonth.length; i++) {
    //indicesForStartingMonth[i] = -1;
    for(int j = 0; j < rows - 1; j++) {
      if(data[i * 2][j] > 0 || data[i * 2 + 1][j] > 0) {
        
        indicesForStartingMonth[i] = ((j - 1) >= 0)? j - 1 : 0;
        startDate[i] = table.getString(j + 1, 0) + "/" + table.getString(j + 1, 1);
        rotationAnglesAtEachStartingPoint[i] = ((360.0) * (indicesForStartingMonth[i]) / months) * PI / 180.0;
        rotationAnglesAtEachStartingPoint[i] -= PI;
        
        println(i  + " " +startDate[i] + " "+  titles[i * 2]+ " : " + j + " " + rotationAnglesAtEachStartingPoint[i]);
        
        break;
      }
    }
  }
  
 
  linePositions = new ArrayList<PVector>(months);
  
  if(height < width)
    radius = ((float)height - constantVerticalMargin * 2.0) / (3.0 * PI + 2.0);
  else 
    radius = ((float)width - constantHorizontalMargin * 2.0) / (3.0 * PI + 2.0);
    
  lineSegmentLength = (2.0 * PI * radius) / (float)months;
  barMaxHeight = lineSegmentLength * 10.0;
  
  int j = (columns - 2) / 2 - 1;
  for(int i = 0; i < months; i++) {
    float x = constantHorizontalMargin + lineSegmentLength / 2.0;
    float y = constantVerticalMargin + lineSegmentLength * (float)i;
    linePositions.add(new PVector(x, y));
  }
}

void drawLine(PVector start, PVector end) {
  line(start.x, start.y, end.x, end.y);
}

void draw() {
  background(bgColor);
  
  PVector startPoint = new PVector();
  
  pushMatrix();
  translate(width / 2.0, constantVerticalMargin + (2 * PI + 1) * radius);
  noFill();
  stroke(255);

  arc(0, 0, radius * 2, radius * 2, -PI, -PI + ((360.0) * (months - padding_months) / months) * PI / 180.0);
  
  fill(255);
  textAlign(CENTER, CENTER);
  textSize(15);
  text("Timeline \nof Issue Date", 0, 0);
  
  for(int i = 0; i < indicesForStartingMonth.length; i++) {
    
    int startMonthIndex = indicesForStartingMonth[i];
    
    pushMatrix();
    rotate(rotationAnglesAtEachStartingPoint[i]);
    
    startPoint.x = radius + padding_months * lineSegmentLength;
    startPoint.y = 0;
    
    float dottedLineX = radius;
    float dotSpace = 4;
    stroke(250);
    while(dottedLineX + dotSpace <= startPoint.x) {
      point(dottedLineX, 0);
      dottedLineX += dotSpace;
    }
    
    pushMatrix();
    fill(255);
    textSize(12);
    if(rotationAnglesAtEachStartingPoint[i] < -PI * 0.5 || rotationAnglesAtEachStartingPoint[i] > PI * 0.5) {
      rotate(PI);
      textAlign(CENTER, BOTTOM);
      text(startDate[i], -startPoint.x + padding_months * lineSegmentLength * 0.5 , -dotSpace);
    } else {
      textAlign(CENTER, BOTTOM);
      text(startDate[i], radius + padding_months * lineSegmentLength * 0.5 , -dotSpace);
    }
    popMatrix();
  
    textSize(10);
    textAlign(LEFT, BOTTOM);
    for(; startMonthIndex < months - 1 - padding_months; startMonthIndex++) {
      PVector endPoint = new PVector(startPoint.x + lineSegmentLength, startPoint.y);
      
      if(startMonthIndex % 12 == 0 && startMonthIndex / 12 == 11 ) {
        pushMatrix();
        fill(255);
        stroke(250);
        if(rotationAnglesAtEachStartingPoint[i] < -PI * 0.5 || rotationAnglesAtEachStartingPoint[i] > PI * 0.5) {
          rotate(PI);
          textAlign(RIGHT, BOTTOM);
          line(-startPoint.x, startPoint.y, -startPoint.x, startPoint.y + barMaxHeight * 0.8 - 2);
          text(startMonthIndex / 12 + 2006, -startPoint.x - 3, startPoint.y + barMaxHeight * 0.8);
          
        } else {
          textAlign(LEFT, BOTTOM);
          line(startPoint.x, startPoint.y, startPoint.x, startPoint.y + barMaxHeight * 0.8 - 2);
          text(startMonthIndex / 12 + 2006, startPoint.x + 2, startPoint.y + barMaxHeight * 0.8);
        }
        
        popMatrix();
      }
      
      if(maxValues[i * 2] > 0) {
        float checkout = data[i * 2][startMonthIndex + 1];
        float rectOSTHeight = map(checkout, minValues[i * 2], maxValues[i * 2], 0, barMaxHeight);
        stroke(200, 200, 200, 200);

        float ratio = rectOSTHeight / barMaxHeight;
        fill(228 * ratio, 86 * ratio, 65 * ratio); 
        rect(startPoint.x, startPoint.y, lineSegmentLength, rectOSTHeight);
      }
      
      if(maxValues[i * 2 + 1] > 0) {
        float rectMovieHeight = map(data[i * 2 + 1][startMonthIndex + 1], minValues[i * 2 + 1], maxValues[i * 2 + 1], 0, barMaxHeight);

        float ratio = rectMovieHeight / barMaxHeight;
        fill(241 * ratio, 169 * ratio, 78 * ratio);
        rect(startPoint.x, startPoint.y - rectMovieHeight, lineSegmentLength, rectMovieHeight);
      }
      
      stroke(200);
      drawLine(startPoint, endPoint);
      
      startPoint = endPoint;
    }
    
    fill(0);
    textSize(15);
    pushMatrix();
    if(rotationAnglesAtEachStartingPoint[i] < -PI * 0.5 || rotationAnglesAtEachStartingPoint[i] > PI * 0.5) {
      rotate(PI);
      textAlign(RIGHT, BOTTOM);
      
      float cw = textWidth(titles[i * 2 + 1]);
      if(cw + startPoint.x > 2 * PI * radius)
        startPoint.x -= cw / 2;
      text(titles[i * 2 + 1], -startPoint.x - 2, -15);
    }
    else {
      textAlign(LEFT, BOTTOM);
      text(titles[i * 2 + 1], startPoint.x, -10);
    }
    popMatrix();
    
    popMatrix();
  }
  popMatrix();
  
  fill(255);
  textSize(24);
  textAlign(LEFT, CENTER);
  text("Hans Zimmer's OST and Its Original Film Checkout Trend", constantHorizontalMargin, constantVerticalMargin / 2);
  fill(200);
  textSize(20);
  text("From Seattle Public Library", constantHorizontalMargin, constantVerticalMargin / 2 + 24);
  
  fill(200);
  textSize(15);
  textAlign(RIGHT, CENTER);
  text("Sihwa Park", width - constantHorizontalMargin, height - constantVerticalMargin);
  text("MAT259, Winter 2018", width - constantHorizontalMargin, height - constantVerticalMargin + 20);
  
  fill(255);
  textSize(15);
  textAlign(LEFT, TOP);
  text("DVD", constantHorizontalMargin + 10, constantVerticalMargin + 100);
  textAlign(LEFT, BOTTOM);
  text("OST", constantHorizontalMargin + 10, constantVerticalMargin + 100);
  
  stroke(200, 200, 200, 200);
  for(int i = 0; i < 20; i++) {
    float barHeight= barMaxHeight * i / 20 + 0.1;
    float ratio = barHeight / barMaxHeight;
    fill(241 * ratio, 169 * ratio, 78 * ratio);
    rect(constantHorizontalMargin + 50 + i * lineSegmentLength, constantVerticalMargin + 100, lineSegmentLength, barHeight);
    fill(228 * ratio, 86 * ratio, 65 * ratio); 
    rect(constantHorizontalMargin + 50 + i * lineSegmentLength, constantVerticalMargin + 100 - barHeight, lineSegmentLength, barHeight);
  }
  
  fill(255);
  textSize(12);
  textAlign(LEFT, CENTER);
  text("Ratio with each medium's\na max checkout", constantHorizontalMargin + 60 + 20 * lineSegmentLength, constantVerticalMargin + 100);
}